// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

package frc.robot;

import java.util.Map;

import edu.wpi.first.wpilibj.shuffleboard.BuiltInWidgets;
import edu.wpi.first.wpilibj.shuffleboard.Shuffleboard;
import edu.wpi.first.wpilibj.shuffleboard.ShuffleboardTab;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.button.CommandXboxController;
import frc.robot.constants.OIConstants;
import frc.robot.constants.ShooterConstants;
import frc.robot.subsystems.Drivetrain;
import frc.robot.subsystems.Indexer;
import frc.robot.subsystems.Intake;
import frc.robot.subsystems.Shooter;

public class RobotContainer {
  private Drivetrain drivetrain;
  private Indexer indexer;
  private Intake intake;
  private Shooter shooter;

  private CommandXboxController driver;

  public RobotContainer() {
    drivetrain = new Drivetrain();
    indexer = new Indexer();
    intake = new Intake();
    shooter = new Shooter();

    driver = new CommandXboxController(OIConstants.kDriverUSB);

    configureBindings();
    configureShuffleboard();
  }

  private void configureBindings() {
    drivetrain.setDefaultCommand(
      drivetrain.driveArcade(
        null, 
        null
      )
    );

    indexer.setDefaultCommand(indexer.stopIndexer());
    intake.setDefaultCommand(intake.stopIntake());
    shooter.setDefaultCommand(shooter.stopShooter());

    //driver inputs//
    driver.rightTrigger().whileTrue(
      shooter.runShooter(driver.getRightTriggerAxis())
    );
    
    driver.a().whileTrue(
      intake.runIntake(1)
      .alongWith(indexer.runIndexer(1))
    );

    driver.b().whileTrue(
      intake.runIntake(-1)
      .alongWith(indexer.runIndexer(-1))
    );

    driver.x().onTrue(smartShooter());
  }

  private void configureShuffleboard() {
    ShuffleboardTab robotIndicatorTab = Shuffleboard.getTab(OIConstants.kRobotIndicatorsTabName);

    robotIndicatorTab.addDouble("Left Drivetrain Encoder Distance", drivetrain::getLeftEncoder)
      .withPosition(0, 0)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);

      robotIndicatorTab.addDouble("Right Drivetrain Encoder Distance", drivetrain::getRightEncoder)
      .withPosition(2, 0)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);

      robotIndicatorTab.addDouble("Drivetrain Gyro", drivetrain::getGyroAngle)
      .withPosition(0, 1)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kGyro);

      robotIndicatorTab.addDouble("Top Shooter Encoder Rate", shooter::getFrontEncoder)
      .withPosition(4, 0)
      .withSize(2, 2)
      .withWidget(BuiltInWidgets.kDial)
      .withProperties(Map.of(
        "Max", ShooterConstants.kShooterFireSpeed)
      );

      robotIndicatorTab.addDouble("Bottom Shooter Encoder Rate", shooter::getFrontEncoder)
      .withPosition(6, 0)
      .withSize(2, 2)
      .withWidget(BuiltInWidgets.kDial)
      .withProperties(Map.of(
        "Max", ShooterConstants.kShooterFireSpeed)
      );
  }

  public Command getAutonomousCommand() {
    return drivetrain.moveDistance(1.5, 5)
    .andThen(drivetrain.goToAngle(45, 2))
    .andThen(shooter.runAutoShooter())
      .until(shooter::shooterAtSpeed)
    .andThen(shooter.runAutoShooter())
      .alongWith(indexer.runIndexer(1))
        .withTimeout(2);
  }

  //smart commands//
  private Command smartShooter() {
    return shooter.runAutoShooter()
      .until(shooter::shooterAtSpeed)
    .andThen(shooter.runAutoShooter())
      .alongWith(indexer.runIndexer(1))
        .withTimeout(2);
  }
}